/*---------------------------------------------------------------------------*
 |          Copyright (c) 2002-2016 PDFlib GmbH. All rights reserved.        |
 +---------------------------------------------------------------------------+
 |    This software may not be copied or distributed except as expressly     |
 |    authorized by PDFlib GmbH's general license agreement or a custom      |
 |    license agreement signed by PDFlib GmbH.                               |
 |    For more information about licensing please refer to www.pdflib.com.   |
 *---------------------------------------------------------------------------*/

/* $Id: ploplibdl.c,v 1.3.2.2 2017/01/11 09:02:57 stm Exp $
 *
 * C wrapper code for dynamically loading the PLOP DLL at runtime.
 *
 * This module is not supported on all platforms.
 *
 */

#include <stdlib.h>
#include <stdio.h>

#include "ploplibdl.h"

/* enable this to avoid error messages */
/*
#define PDF_SILENT
*/

#ifdef WIN32

#define PLOP_DLLNAME                     "libplop.dll"

/* ---------------------------------- MVS ----------------------------- */

#elif defined(__MVS__)

#define PLOP_DLLNAME                     "PLOP"

/* --------------------- Linux and other Unixes -----------------------------*/

#elif defined(__linux__) || defined(linux) \
    || defined (__FreeBSD__) || defined (__HOS_AIX__) || defined(__hpux)

#define PLOP_DLLNAME                     "libplop.so"

/* ---------------------------------- Mac OS X  ----------------------------- */

#elif defined(__APPLE__)

#define PLOP_DLLNAME                     "libplop.dylib"

/* ---------------------------------- AS/400  ----------------------------- */

#elif defined __ILEC400__

#define PLOP_DLLNAME                     "PLOP"

/* ---------------------------------- unknown  ----------------------------- */

#else

#error No DLL loading code for this platform available!

#endif

#include "dl_dl.h"

static void
plop_dlerror(const char *msg)
{
#ifndef PDF_SILENT
    fputs(msg, stderr);
#endif
}

/* Load the PLOP DLL and fetch the API structure */
PDFLIB_API const PLOP_api * PDFLIB_CALL
PLOP_new_dl(PLOP **p_plop_obj)
{
    const PLOP_api *plop_api, *(PDFLIB_CALL *get_api)(void);
    char buf[256];

    /* load the PLOP DLL... */
    void *handle = pdfdl_dlopen(PLOP_DLLNAME);

    if (!handle)
    {
	plop_dlerror("Error: couldn't load PLOP DLL\n");
	return NULL;
    }

    /* ...and fetch function pointers */
    *(void**) (&get_api) = pdfdl_dlsym(handle, "PLOP_get_api");

    if (get_api == NULL)
    {
	plop_dlerror(
	    "Error: couldn't find function PLOP_get_api in PLOP DLL\n");
	pdfdl_dlclose(handle);
	return NULL;
    }

    /* Fetch the API structure. */
    plop_api = (*get_api)();

    /*
     * Check the version number of the loaded DLL against that of
     * the included header file to avoid version mismatch.
     */
    if (plop_api->sizeof_PLOP_api != sizeof(PLOP_api) ||
            plop_api->major != PLOP_MAJORVERSION ||
            plop_api->minor != PLOP_MINORVERSION) {
	sprintf(buf,
	"Error: loaded wrong version of PLOP DLL\n"
	"Expected version %d.%d (API size %lu), loaded %d.%d (API size %lu)\n",
	PLOP_MAJORVERSION, PLOP_MINORVERSION, (unsigned long) sizeof(PLOP_api),
	plop_api->major, plop_api->minor,
	(unsigned long) plop_api->sizeof_PLOP_api);
	plop_dlerror(buf);
	pdfdl_dlclose(handle);
	return NULL;
    }

    /*
     * Create a new PLOP object; use PLOP_new2() so that we can store
     * the DLL handle within PLOP and later retrieve it.
     */
    if ((*p_plop_obj = plop_api->PLOP_new2(NULL, handle)) == (PLOP *) NULL)
    {
        plop_dlerror("Couldn't create PLOP object (out of memory)!\n");
        pdfdl_dlclose(handle);
        return NULL;
    }

    return plop_api;
}

/* delete the PLOP object and unload the previously loaded PLOP DLL */
PDFLIB_API void PDFLIB_CALL
PLOP_delete_dl(const PLOP_api *plop_api, PLOP *plop)
{
    if (plop_api && plop)
    {
        /* fetch the DLL handle (previously stored in PDFlib) */
        void *handle = plop_api->PLOP_get_opaque(plop);

        plop_api->PLOP_delete(plop);

        if (handle)
            pdfdl_dlclose(handle);
    }
}
