/* $Id: multisign.cpp,v 1.4 2016/03/24 13:27:12 rp Exp $
 *
 * PDFlib PLOP DS:
 * Demonstrate mass signature with optional PKCS#11 session handling
 */

#include <iostream>

#include "plop.hpp"

using namespace std;
using namespace pdflib;

namespace
{
wstring get_wstring(const PLOP& plop, const string& utf8_string);
}

int
main(int argc, char **argv)
{
    PLOP plop;

    /* Basic signature options */
    static const wstring sign_opts =
        L"ltv=try "          /* use ltv=full to ensure LTV status */
#if defined(USE_PKCS11)
        L"engine=pkcs#11 digitalid={filename=cryptoki.dll}"
#else
        L"engine=builtin digitalid={filename=demo_signer_rsa_2048.p12} password={demo}"
#endif
        ;

    /* This is where input files live. Adjust as necessary. */
    const wstring searchpath(L"../data");

    if (argc < 2)
    {
        wcerr << L"usage: multisign <infile>..." << endl;
        return 2;
    }

    try {
	int i;
	wostringstream optlist;

	optlist << L"searchpath={" << searchpath << L"}";
	plop.set_option(optlist.str());

	if (plop.prepare_signature(sign_opts) == -1)
	{
	    wcerr << L"Error: " << plop.get_apiname() << L": "
	                        << plop.get_errmsg() << endl;
            return 2;
	}

	for (i = 1; i < argc; i++)
	{
            int doc;
	    wostringstream signed_name;

	    const wstring doc_name(get_wstring(plop, string(argv[i])));

	    wcout << L"Signing '" << doc_name  << "'..." << endl;

            if ((doc = plop.open_document(doc_name, L"")) == -1)
            {
                wcerr << L"Error: Document '" << doc_name << ": "
                    << plop.get_apiname() << L": " << plop.get_errmsg() << endl;
		continue;
            }

            signed_name << doc_name << L"-signed.pdf";

            optlist.str(L"");
            optlist << L"input=" << doc;
            if (plop.create_document(signed_name.str(), optlist.str()) == -1)
            {
                wcerr << L"Error: Output document '" << signed_name.str()
                    << ": " << plop.get_apiname() << L": "
                    << plop.get_errmsg() << endl;


#if defined(USE_PKCS11)
		/* The PKCS#11 session may have expired;
		 * restart and try again;
		 */
		if (plop.prepare_signature(sign_opts) == -1)
		{
                    wcerr << L"Error: " << plop.get_apiname() << L": "
                                        << plop.get_errmsg() << endl;
		    return 2;
		}
		if (plop.create_document(signed_name, 0, optlist.str()) == -1)
		{
                    wcerr << L"Error: Output document '" << signed_name.str()
                        << ": " << plop.get_apiname() << L": "
                        << plop.get_errmsg() << endl;
		    return 2;
		}
#endif
            }

            plop.close_document(doc, L"");
	}
    }
    catch (PLOP::Exception &ex) {
        wcerr << L"PLOP exception occurred in multisign:" << endl;
        wcerr << L"[" << ex.get_errnum() << L"] " << ex.get_apiname()
                << L": " << ex.get_errmsg() << endl;
        return 99;
    }

    return 0;
}

namespace
{

/*
 * Get a wstring for the given string, assuming the string is encoded in
 * UTF-8.
 */
wstring get_wstring(const PLOP& plop, const string& utf8_string)
{
    const size_t size = sizeof(wstring::value_type);
    string wide_string;

    switch (size)
    {
    case 2:
        wide_string = plop.convert_to_unicode(L"utf8",
                            utf8_string, L"outputformat=utf16");
        break;

    case 4:
        wide_string = plop.convert_to_unicode(L"utf8",
                            utf8_string, L"outputformat=utf32");
        break;

    default:
        throw std::logic_error("Unsupported wchar_t size");
    }

    return wstring(reinterpret_cast<const wchar_t *>(wide_string.data()),
            wide_string.length() / size);
}

} // end anonymous namespace
